;; Generated package description from num3-mode.el  -*- no-byte-compile: t -*-
(define-package "num3-mode" "1.3.0.20201128.235916" "highlight groups of digits in long numbers" 'nil :url "https://elpa.gnu.org/packages/num3-mode.html" :keywords '("faces" "minor-mode") :authors '(("Felix Lee <felix8a@gmail.com>, Michal Nazarewicz" . "mina86@mina86.com")) :maintainer '("Michal Nazarewicz" . "mina86@mina86.com"))
